@@ -2,9 +2,12 @@
 ;;; Copyright © 2015, 2016 Ludovic Courtès <ludo@gnu.org>
 ;;; Copyright © 2015 Tomáš Čech <sleep_walker@gnu.org>
 ;;; Copyright © 2016 Leo Famulari <leo@famulari.name>
-;;; Copyright © 2016 Ricardo Wurmus <rekado@elephly.net>
+;;; Copyright © 2016, 2017 Ricardo Wurmus <rekado@elephly.net>
 ;;; Copyright © 2016 Efraim Flashner <efraim@flashner.co.il>
 ;;; Copyright © 2016 Andreas Enge <andreas@enge.fr>
+;;; Copyright © 2017 Manolis Fragkiskos Ragkousis <manolis837@gmail.com>
+;;; Copyright © 2017 Ben Woodcroft <donttrustben@gmail.com>
+;;; Copyright © 2017 Tobias Geerinckx-Rice <me@tobias.gr>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -23,10 +26,11 @@
 
 (define-module (gnu packages graphics)
   #:use-module (guix download)
-  #:use-module (guix svn-download)
+  #:use-module (guix git-download)
   #:use-module (guix packages)
   #:use-module (guix build-system gnu)
   #:use-module (guix build-system cmake)
+  #:use-module (guix build-system python)
   #:use-module ((guix licenses) #:prefix license:)
   #:use-module (gnu packages)
   #:use-module (gnu packages algebra)
@@ -35,11 +39,13 @@
   #:use-module (gnu packages bash)
   #:use-module (gnu packages bison)
   #:use-module (gnu packages boost)
+  #:use-module (gnu packages check)
   #:use-module (gnu packages documentation)
   #:use-module (gnu packages haskell)
   #:use-module (gnu packages image)
   #:use-module (gnu packages python)
   #:use-module (gnu packages flex)
+  #:use-module (gnu packages fonts)
   #:use-module (gnu packages fontutils)
   #:use-module (gnu packages pkg-config)
   #:use-module (gnu packages pulseaudio)  ;libsndfile, libsamplerate
@@ -54,10 +60,12 @@
   #:use-module (gnu packages image)
   #:use-module (gnu packages jemalloc)
   #:use-module (gnu packages photo)
+  #:use-module (gnu packages pth)
   #:use-module (gnu packages python)
   #:use-module (gnu packages qt)
   #:use-module (gnu packages readline)
   #:use-module (gnu packages sdl)
+  #:use-module (gnu packages swig)
   #:use-module (gnu packages video)
   #:use-module (gnu packages xml)
   #:use-module (gnu packages xorg))
@@ -65,14 +73,14 @@
 (define-public blender
   (package
     (name "blender")
-    (version "2.76b")
+    (version "2.79")
     (source (origin
               (method url-fetch)
-              (uri (string-append "http://download.blender.org/source/"
+              (uri (string-append "https://download.blender.org/source/"
                                   "blender-" version ".tar.gz"))
               (sha256
                (base32
-                "0pb0mlj4vj0iir528ifqq67nsh3ca1942933d9cwlbpcja2jm1dx"))))
+                "16f84mdzkmwjmqahjj64kbyk4kagdj4mcr8qjazs1952d7kh7pm9"))))
     (build-system cmake-build-system)
     (arguments
      `(;; Test files are very large and not included in the release tarball.
@@ -129,7 +137,7 @@
        ("openal" ,openal)
        ("python" ,python-wrapper)
        ("zlib" ,zlib)))
-    (home-page "http://blender.org/")
+    (home-page "https://blender.org/")
     (synopsis "3D graphics creation suite")
     (description
      "Blender is a 3D graphics creation suite.  It supports the entirety of
@@ -138,6 +146,32 @@ compositing and motion tracking, even video editing and game creation.  The
 application can be customized via its API for Python scripting.")
     (license license:gpl2+)))
 
+(define-public assimp
+  (package
+    (name "assimp")
+    (version "3.3.1")
+    (source (origin
+              (method url-fetch)
+              (uri (string-append "https://github.com/assimp/assimp/archive/v"
+                                  version ".tar.gz"))
+              (file-name (string-append name "-" version ".tar.gz"))
+              (sha256
+               (base32
+                "1gy7zlgkf4nmyv8n674p3f30asis0gjz8icyy11i693n13ww71fk"))))
+    (build-system cmake-build-system)
+    (inputs
+     `(("zlib" ,zlib)))
+    (home-page "http://assimp.org/")
+    (synopsis "Asset import library")
+    (description
+     "The Open Asset Import Library loads more than 40 3D file formats into
+one unified data structure.  Additionally, assimp features various mesh post
+processing tools: normals and tangent space generation, triangulation, vertex
+cache locality optimization, removal of degenerate primitives and duplicate
+vertices, sorting by primitive type, merging of redundant materials and many
+more.")
+    (license license:bsd-3)))
+
 (define-public cgal
   (package
     (name "cgal")
@@ -200,6 +234,74 @@ quaternions and other useful 2D and 3D math functions.  Iex is an
 exception-handling library.")
     (license license:bsd-3)))
 
+(define-public ogre
+  (package
+    (name "ogre")
+    (version "1.10.7")
+    (source
+     (origin
+       (method url-fetch)
+       (uri (string-append "https://github.com/OGRECave/" name
+                           "/archive/v" version ".tar.gz"))
+       (sha256
+        (base32
+         "1p0c91cc7zg3c00wjaibnxb0a0xm14mkg0h65pzpw93m0d6nc8wd"))
+       (file-name (string-append name "-" version ".tar.gz"))))
+    (build-system cmake-build-system)
+    (arguments
+     '(#:phases
+       (modify-phases %standard-phases
+         (add-before 'configure 'pre-configure
+           (lambda _
+             ;; It expects googletest source to be downloaded and
+             ;; be in a specific place.
+             (substitute* "Tests/CMakeLists.txt"
+               (("URL(.*)$" _ suffix)
+                (string-append "URL " suffix
+                               "\t\tURL_HASH "
+                               "MD5=16877098823401d1bf2ed7891d7dce36\n")))
+             #t))
+         (add-before 'build 'pre-build
+           (lambda* (#:key inputs #:allow-other-keys)
+             (copy-file (assoc-ref inputs "googletest-source")
+                        (string-append (getcwd)
+                                       "/Tests/googletest-prefix/src/"
+                                       "release-1.8.0.tar.gz"))
+             #t)))
+       #:configure-flags
+       (list "-DOGRE_BUILD_TESTS=TRUE"
+             (string-append "-DCMAKE_INSTALL_RPATH="
+                            (assoc-ref %outputs "out") "/lib:"
+                            (assoc-ref %outputs "out") "/lib/OGRE:"
+                            (assoc-ref %build-inputs "googletest") "/lib")
+             "-DOGRE_INSTALL_DOCS=TRUE"
+             "-DOGRE_INSTALL_SAMPLES=TRUE"
+             "-DOGRE_INSTALL_SAMPLES_SOURCE=TRUE")))
+    (native-inputs
+     `(("boost" ,boost)
+       ("doxygen" ,doxygen)
+       ("googletest-source" ,(package-source googletest))
+       ("pkg-config" ,pkg-config)))
+    (inputs
+     `(("font-dejavu" ,font-dejavu)
+       ("freeimage" ,freeimage)
+       ("freetype" ,freetype)
+       ("glu" ,glu)
+       ("googletest" ,googletest)
+       ("sdl2" ,sdl2)
+       ("libxaw" ,libxaw)
+       ("libxrandr" ,libxrandr)
+       ("tinyxml" ,tinyxml)
+       ("zziplib" ,zziplib)))
+    (synopsis "Scene-oriented, flexible 3D engine written in C++")
+    (description
+     "OGRE (Object-Oriented Graphics Rendering Engine) is a scene-oriented,
+flexible 3D engine written in C++ designed to make it easier and more intuitive
+for developers to produce applications utilising hardware-accelerated 3D
+graphics.")
+    (home-page "http://www.ogre3d.org/")
+    (license license:expat)))
+
 (define-public openexr
   (package
     (name "openexr")
@@ -291,6 +393,51 @@ visual effects work for film.")
     (home-page "http://www.openimageio.org")
     (license license:bsd-3)))
 
+(define-public openscenegraph
+  (package
+    (name "openscenegraph")
+    (version "3.4.0")
+    (source
+     (origin
+       (method url-fetch)
+       (uri (string-append "http://trac.openscenegraph.org/downloads/developer_releases/"
+                           "OpenSceneGraph-" version ".zip"))
+       (sha256
+        (base32
+         "03h4wfqqk7rf3mpz0sa99gy715cwpala7964z2npd8jxfn27swjw"))
+       (patches (search-patches "openscenegraph-ffmpeg3.patch"))
+       (file-name (string-append name "-" version ".zip"))))
+    (build-system cmake-build-system)
+    (arguments
+     `(#:tests? #f ;; No test target available.
+       #:configure-flags
+       (list (string-append "-DCMAKE_INSTALL_RPATH="
+                            (assoc-ref %outputs "out") "/lib:"
+                            (assoc-ref %outputs "out") "/lib64")
+             ;; We need to set this flag or otherwise 'rd' will be added
+             ;; to the name of the library binaries and break linking
+             ;; with other programs.
+             "-DCMAKE_BUILD_TYPE=Release")))
+    (native-inputs
+     `(("unzip" ,unzip)))
+    (inputs
+     `(("giflib", giflib)
+       ("jasper", jasper)
+       ("librsvg", librsvg)
+       ("pth", pth)
+       ("qtbase", qtbase)
+       ("ffmpeg", ffmpeg)
+       ("mesa", mesa)))
+    (synopsis "High performance real-time graphics toolkit")
+    (description
+     "The OpenSceneGraph is a high performance 3D graphics toolkit
+used by application developers in fields such as visual simulation, games,
+virtual reality, scientific visualization and modeling.")
+    (home-page "http://www.openscenegraph.org")
+    ;; The 'LICENSE' file explains that the source is licensed under
+    ;; LGPL 2.1, but with 4 exceptions. This version is called OSGPL.
+    (license license:lgpl2.1)))
+
 (define-public rapicorn
   (package
     (name "rapicorn")
@@ -354,7 +501,7 @@ visual effects work for film.")
        ("intltool" ,intltool)
        ("pkg-config" ,pkg-config)
        ("xvfb" ,xorg-server)))
-    (home-page "http://rapicorn.org")
+    (home-page "https://rapicorn.testbit.org/")
     (synopsis "Toolkit for rapid development of user interfaces")
     (description
      "Rapicorn is a toolkit for rapid development of user interfaces in C++
@@ -396,66 +543,66 @@ output.")
     (license (license:non-copyleft "file://LICENSE"))))
 
 (define-public brdf-explorer
-  (package
-    (name "brdf-explorer")
-    (version "17")                                ;svn revision
-    (source (origin
-              ;; There are no release tarballs, and not even tags in the repo,
-              ;; so use the latest revision.
-              (method svn-fetch)
-              (uri (svn-reference
-                    (url "http://github.com/wdas/brdf")
-                    (revision (string->number version))))
-              (sha256
-               (base32
-                "1458fwsqxramh0gpnp24x7brfpl9afhvr1wqg6c78xqwf32960m5"))
-              (file-name (string-append name "-" version "-checkout"))))
-    (build-system gnu-build-system)
-    (arguments
-     `(#:phases (modify-phases %standard-phases
-                  (replace 'configure
-                           (lambda* (#:key outputs #:allow-other-keys)
-                             (let ((out (assoc-ref outputs "out")))
-                               (chdir "trunk")
-                               (zero? (system* "qmake"
-                                               (string-append
-                                                "prefix=" out))))))
-                  (add-after 'install 'wrap-program
-                             (lambda* (#:key outputs #:allow-other-keys)
-                               (let* ((out (assoc-ref outputs "out"))
-                                      (bin (string-append out "/bin"))
-                                      (data (string-append
-                                             out "/share/brdf")))
-                                 (with-directory-excursion bin
-                                   (rename-file "brdf" ".brdf-real")
-                                   (call-with-output-file "brdf"
-                                     (lambda (port)
-                                       (format port "#!/bin/sh
+  ;; There are no release tarballs, and not even tags in the repo,
+  ;; so use the latest revision.
+  (let ((commit "5b2cd46f38a06e47207fa7229b72d37beb945019")
+        (revision "1"))
+    (package
+      (name "brdf-explorer")
+      (version (string-append "1.0.0-" revision "." (string-take commit 9)))
+      (source (origin
+                (method git-fetch)
+                (uri (git-reference
+                      (url "https://github.com/wdas/brdf.git")
+                      (commit commit)))
+                (sha256
+                 (base32
+                  "06vzbiajzbi2xl8jlff5d45bc9wd68i3jdndfab1f3jgfrd8bsgx"))
+                (file-name (string-append name "-" version "-checkout"))))
+      (build-system gnu-build-system)
+      (arguments
+       `(#:phases (modify-phases %standard-phases
+                    (replace 'configure
+                      (lambda* (#:key outputs #:allow-other-keys)
+                        (let ((out (assoc-ref outputs "out")))
+                          (zero? (system* "qmake"
+                                          (string-append "prefix=" out))))))
+                    (add-after 'install 'wrap-program
+                      (lambda* (#:key outputs #:allow-other-keys)
+                        (let* ((out (assoc-ref outputs "out"))
+                               (bin (string-append out "/bin"))
+                               (data (string-append
+                                      out "/share/brdf")))
+                          (with-directory-excursion bin
+                            (rename-file "brdf" ".brdf-real")
+                            (call-with-output-file "brdf"
+                              (lambda (port)
+                                (format port "#!/bin/sh
 # Run the thing from its home, otherwise it just bails out.
 cd \"~a\"
 exec -a \"$0\" ~a/.brdf-real~%"
-                                               data bin)))
-                                   (chmod "brdf" #o555))))))))
-    (native-inputs
-     `(("qt" ,qt-4)))                             ;for 'qmake'
-    (inputs
-     `(("qt" ,qt-4)
-       ("mesa" ,mesa)
-       ("glew" ,glew)
-       ("freeglut" ,freeglut)
-       ("zlib" ,zlib)))
-    (home-page "http://www.disneyanimation.com/technology/brdf.html")
-    (synopsis
-     "Analyze bidirectional reflectance distribution functions (BRDFs)")
-    (description
-     "BRDF Explorer is an application that allows the development and analysis
+                                        data bin)))
+                            (chmod "brdf" #o555))))))))
+      (native-inputs
+       `(("qttools" ,qttools))) ;for 'qmake'
+      (inputs
+       `(("qtbase" ,qtbase)
+         ("mesa" ,mesa)
+         ("glew" ,glew)
+         ("freeglut" ,freeglut)
+         ("zlib" ,zlib)))
+      (home-page "http://www.disneyanimation.com/technology/brdf.html")
+      (synopsis
+       "Analyze bidirectional reflectance distribution functions (BRDFs)")
+      (description
+       "BRDF Explorer is an application that allows the development and analysis
 of bidirectional reflectance distribution functions (BRDFs).  It can load and
 plot analytic BRDF functions (coded as functions in OpenGL's GLSL shader
 language), measured material data from the MERL database, and anisotropic
 measured material data from MIT CSAIL.  Graphs and visualizations update in
 real time as parameters are changed, making it a useful tool for evaluating
 and understanding different BRDFs (and other component functions).")
-    (license license:ms-pl)))
+      (license license:ms-pl))))
 
 (define-public agg
   (package
@@ -484,13 +631,12 @@ and understanding different BRDFs (and other component functions).")
              (string-append "--x-libraries=" (assoc-ref %build-inputs "libx11")
                             "/lib"))
        #:phases
-       (alist-cons-after
-        'unpack 'autoreconf
-        (lambda _
-          ;; let's call configure from configure phase and not now
-          (substitute* "autogen.sh" (("./configure") "# ./configure"))
-          (zero? (system* "sh" "autogen.sh")))
-        %standard-phases)))
+       (modify-phases %standard-phases
+         (add-after 'unpack 'autoreconf
+           (lambda _
+             ;; let's call configure from configure phase and not now
+             (substitute* "autogen.sh" (("./configure") "# ./configure"))
+             (zero? (system* "sh" "autogen.sh")))))))
     (native-inputs
      `(("pkg-config" ,pkg-config)
        ("libtool" ,libtool)
@@ -510,3 +656,33 @@ and understanding different BRDFs (and other component functions).")
 It supports sub-pixel resolutions and anti-aliasing.  It is also library for
 rendering SVG graphics.")
     (license license:gpl2+)))
+
+(define-public python-pastel
+  (package
+    (name "python-pastel")
+    (version "0.1.0")
+    (source
+     (origin
+       (method url-fetch)
+       (uri (pypi-uri "pastel" version))
+       (sha256
+        (base32
+         "1hqbm934n5yjwn31aq8h7shrr0rcy326wrqfc856vyn0gr0sy21i"))))
+    (build-system python-build-system)
+    (native-inputs
+     `(("python-pytest" ,python-pytest)))
+    (home-page "https://github.com/sdispater/pastel")
+    (synopsis "Library to colorize strings in your terminal")
+    (description "Pastel is a simple library to help you colorize strings in
+your terminal.  It comes bundled with predefined styles:
+@enumerate
+@item info: green
+@item comment: yellow
+@item question: black on cyan
+@item error: white on red
+@end enumerate
+")
+    (license license:expat)))
+
+(define-public python2-pastel
+  (package-with-python2 python-pastel))
