@@ -1,5 +1,6 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2015 Sou Bunnbu <iyzsong@gmail.com>
+;;; Copyright © 2017 Efraim Flashner <efraim@flashner.co.il>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -17,27 +18,45 @@
 ;;; along with GNU Guix.  If not, see <http://www.gnu.org/licenses/>.
 
 (define-module (gnu packages jemalloc)
+  #:use-module (srfi srfi-1)
+  #:use-module (srfi srfi-26)
   #:use-module ((guix licenses) #:select (bsd-2))
   #:use-module (guix packages)
   #:use-module (guix download)
-  #:use-module (gnu packages base)
-  #:use-module (gnu packages gcc)
+  #:use-module (gnu packages perl)
   #:use-module (guix build-system gnu))
 
 (define-public jemalloc
   (package
     (name "jemalloc")
-    (version "4.2.0")
+    (version "5.0.1")
     (source (origin
               (method url-fetch)
               (uri (string-append
-                    "http://www.canonware.com/download/jemalloc/"
-                    name "-" version ".tar.bz2"))
+                    "https://github.com/jemalloc/jemalloc/releases/download/"
+                    version "/jemalloc-" version ".tar.bz2"))
               (sha256
                (base32
-                "1jvasihaizawz44j02bri47bd905flns03nkigipys81p6pds5mj"))))
+                "1sf3lzgb0y8nnyzmp4zrca3sngdxw3kfh20sna9z03jv74fph528"))))
     (build-system gnu-build-system)
-    (home-page "http://www.canonware.com/jemalloc/")
+    (arguments
+     `(#:phases
+       (modify-phases %standard-phases
+         (add-after 'unpack 'delete-thp-test
+           ;; This test does not check if transparent huge pages are supported
+           ;; on the system before running the test.
+           (lambda _
+             (substitute* "Makefile.in"
+               (("\\$\\(srcroot\\)test/unit/pages.c \\\\") "\\"))
+             #t)))
+       ,@(if (any (cute string-prefix? <> (or (%current-target-system)
+                                              (%current-system)))
+                 '("x64_64" "i686"))
+           ;; Transparent huge pages are only enabled by default on Intel processors
+           '()
+           '(#:configure-flags (list "--disable-thp")))))
+    (inputs `(("perl" ,perl)))
+    (home-page "http://jemalloc.net/")
     (synopsis "General-purpose scalable concurrent malloc implementation")
     (description
      "This library providing a malloc(3) implementation that emphasizes
