@@ -1,6 +1,10 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2012, 2014 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2012, 2014, 2015, 2017 Ludovic Courtès <ludo@gnu.org>
 ;;; Copyright © 2014 David Thompson <dthompson2@worcester.edu>
+;;; Copyright © 2015, 2016 Ricardo Wurmus <rekado@elephly.net>
+;;; Copyright © 2015 Alex Kost <alezost@gmail.com>
+;;; Copyright © 2015, 2016 Efraim Flashner <efraim@flashner.co.il>
+;;; Copyright © 2017 Tobias Geerinckx-Rice <me@tobias.gr>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -22,18 +26,19 @@
   #:use-module (guix download)
   #:use-module (guix packages)
   #:use-module (guix build-system gnu)
+  #:use-module (gnu packages databases)
   #:use-module (gnu packages flex)
-  #:use-module (gnu packages gdbm)
+  #:use-module (gnu packages gawk)
   #:use-module (gnu packages groff)
   #:use-module (gnu packages less)
-  #:use-module (gnu packages lynx)
   #:use-module (gnu packages perl)
-  #:use-module (gnu packages pkg-config))
+  #:use-module (gnu packages pkg-config)
+  #:use-module (gnu packages linux))
 
 (define-public libpipeline
   (package
     (name "libpipeline")
-    (version "1.3.0")
+    (version "1.4.2")
     (source (origin
               (method url-fetch)
               (uri (string-append
@@ -41,7 +46,7 @@
                     version ".tar.gz"))
               (sha256
                (base32
-                "12d6ldcj7kv2nv832b23v97g7035d0ybq0ig7h0yr7xk9czd3z7i"))))
+                "1gkrfqkphdc6gk8gic68asallj59i3cfq6nd31ppks0cljdgrwgy"))))
     (build-system gnu-build-system)
     (home-page "http://libpipeline.nongnu.org/")
     (synopsis "C library for manipulating pipelines of subprocesses")
@@ -53,55 +58,80 @@ a flexible and convenient way.")
 (define-public man-db
   (package
     (name "man-db")
-    (version "2.6.6")
+    (version "2.7.6.1")
     (source (origin
               (method url-fetch)
               (uri (string-append "mirror://savannah/man-db/man-db-"
                                   version ".tar.xz"))
               (sha256
                (base32
-                "1hv6byj6sg6cp3jyf08gbmdm4pwhvd5hzmb94xl0w7prin6hzabx"))))
+                "0gqgs4zc3r87apns0k5qp689p2ylxx2596s2mkmkxjjay99brv88"))))
     (build-system gnu-build-system)
     (arguments
-     '(#:phases
-       (alist-cons-after
-        'patch-source-shebangs 'patch-test-shebangs
-        (lambda* (#:key outputs #:allow-other-keys)
-          ;; Patch shebangs in test scripts.
-          (let ((out (assoc-ref outputs "out")))
-            (for-each (lambda (file)
-                        (substitute* file
-                          (("#! /bin/sh")
-                           (string-append "#!" (which "sh")))))
-                      (remove file-is-directory?
-                              (find-files "src/tests" ".*")))))
-        %standard-phases)
+     `(#:phases
+       (modify-phases %standard-phases
+         (add-after 'patch-source-shebangs 'patch-test-shebangs
+           (lambda* (#:key outputs #:allow-other-keys)
+             ;; Patch shebangs in test scripts.
+             (let ((out (assoc-ref outputs "out")))
+               (for-each (lambda (file)
+                           (substitute* file
+                             (("#! /bin/sh")
+                              (string-append "#!" (which "sh")))))
+                         (remove file-is-directory?
+                                 (find-files "src/tests" ".*"))))))
+         (add-after 'unpack 'patch-iconv-path
+           (lambda* (#:key inputs #:allow-other-keys)
+             (substitute* "src/man.c"
+               (("\"iconv\"")
+                (string-append "\"" (which "iconv") "\"")))
+             #t)))
        #:configure-flags
        (let ((groff (assoc-ref %build-inputs "groff"))
+             (groff-minimal (assoc-ref %build-inputs "groff-minimal"))
              (less  (assoc-ref %build-inputs "less"))
              (gzip  (assoc-ref %build-inputs "gzip"))
-             (bzip2  (assoc-ref %build-inputs "bzip2"))
-             (xz  (assoc-ref %build-inputs "xz")))
+             (bzip2 (assoc-ref %build-inputs "bzip2"))
+             (xz    (assoc-ref %build-inputs "xz"))
+             (util  (assoc-ref %build-inputs "util-linux")))
          ;; Invoke groff, less, gzip, bzip2, and xz directly from the store.
-         (append (list "--disable-setuid" ;; Disable setuid man user.
+         (append (list ;; Disable setuid man user.
+                       "--disable-setuid"
+                       ;; Don't constrain ownership of system-wide cache files.
+                       ;; Otherwise creating the manpage database fails with
+                       ;; man-db > 2.7.5.
+                       "--disable-cache-owner"
                        (string-append "--with-pager=" less "/bin/less")
                        (string-append "--with-gzip=" gzip "/bin/gzip")
                        (string-append "--with-bzip2=" bzip2 "/bin/gzip")
-                       (string-append "--with-xz=" xz "/bin/xz"))
+                       (string-append "--with-xz=" xz "/bin/xz")
+                       (string-append "--with-col=" util "/bin/col")
+                       ;; Default value is "/usr/lib/tmpfiles.d" (not
+                       ;; prefix-sensitive).
+                       (string-append "--with-systemdtmpfilesdir="
+                                      %output "/lib/tmpfiles.d"))
                  (map (lambda (prog)
-                        (string-append "--with-" prog "=" groff "/bin/" prog))
+                        (string-append "--with-" prog "=" groff-minimal
+                                       "/bin/" prog))
                       '("nroff" "eqn" "neqn" "tbl" "refer" "pic"))))
+
+       ;; At run time we should refer to GROFF-MINIMAL, not GROFF (the latter
+       ;; pulls in Perl.)
+       #:disallowed-references (,groff)
+
        #:modules ((guix build gnu-build-system)
                   (guix build utils)
                   (srfi srfi-1))))
     (native-inputs
-     `(("pkg-config" ,pkg-config)))
+     `(("pkg-config" ,pkg-config)
+       ("groff" ,groff)))   ;needed at build time (troff, grops, soelim, etc.)
     (inputs
      `(("flex" ,flex)
        ("gdbm" ,gdbm)
-       ("groff" ,groff)
+       ("groff-minimal" ,groff-minimal)
        ("less" ,less)
-       ("libpipeline" ,libpipeline)))
+       ("libpipeline" ,libpipeline)
+       ("util-linux" ,util-linux)))
     (native-search-paths
      (list (search-path-specification
             (variable "MANPATH")
@@ -117,18 +147,23 @@ the traditional flat-text whatis databases.")
 (define-public man-pages
   (package
     (name "man-pages")
-    (version "3.69")
+    (version "4.14")
     (source (origin
               (method url-fetch)
-              (uri (string-append
+              (uri
+                (list
+                  (string-append
                     "mirror://kernel.org/linux/docs/man-pages/man-pages-"
-                    version ".tar.xz"))
+                    version ".tar.xz")
+                  (string-append
+                    "mirror://kernel.org/linux/docs/man-pages/Archive/"
+                    "man-pages-" version ".tar.xz")))
               (sha256
                (base32
-                "18zzmdzjihdnyg4vamk0jp6v6826vrsgal3kqqxvfq9bzyrh8xm2"))))
+                "0wf9ymqxk1k5xwcl3n919p66a1aayif3x4cahj4w04y3k1wbhlih"))))
     (build-system gnu-build-system)
     (arguments
-     '(#:phases (alist-delete 'configure %standard-phases)
+     '(#:phases (modify-phases %standard-phases (delete 'configure))
 
        ;; The 'all' target depends on three targets that directly populate
        ;; $(MANDIR) based on its current contents.  Doing that in parallel
@@ -139,7 +174,7 @@ the traditional flat-text whatis databases.")
        #:make-flags (list (string-append "MANDIR="
                                          (assoc-ref %outputs "out")
                                          "/share/man"))))
-    (home-page "http://www.kernel.org/doc/man-pages/")
+    (home-page "https://www.kernel.org/doc/man-pages/")
     (synopsis "Development manual pages from the Linux project")
     (description
      "This package provides traditional Unix \"man pages\" documenting the
@@ -151,7 +186,7 @@ Linux kernel and C library interfaces employed by user-space programs.")
 (define-public help2man
   (package
     (name "help2man")
-    (version "1.46.4")
+    (version "1.47.4")
     (source
      (origin
       (method url-fetch)
@@ -159,7 +194,7 @@ Linux kernel and C library interfaces employed by user-space programs.")
                           version ".tar.xz"))
       (sha256
        (base32
-        "0csn7jx7nhlrflalw1992p3l5afawlpdyjdff2q5bk5hadgz3rqs"))))
+        "0lvp4306f5nq08f3snffs5pp1zwv8l35z6f5g0dds51zs6bzdv6l"))))
     (build-system gnu-build-system)
     (arguments `(;; There's no `check' target.
                  #:tests? #f))
@@ -167,12 +202,37 @@ Linux kernel and C library interfaces employed by user-space programs.")
      `(("perl" ,perl)
        ;; TODO: Add these optional dependencies.
        ;; ("perl-LocaleGettext" ,perl-LocaleGettext)
-       ;; ("gettext" ,gnu-gettext)
+       ;; ("gettext" ,gettext-minimal)
        ))
-    (home-page "http://www.gnu.org/software/help2man/")
+    (home-page "https://www.gnu.org/software/help2man/")
     (synopsis "Automatically generate man pages from program --help")
     (description
      "GNU help2man is a program that converts the output of standard
 \"--help\" and \"--version\" command-line arguments into a manual page
 automatically.")
     (license gpl3+)))
+
+(define-public txt2man
+  (package
+    (name "txt2man")
+    (version "1.6.0")
+    (source
+     (origin
+       (method url-fetch)
+       (uri (string-append
+             "https://github.com/mvertes/txt2man/archive/txt2man-"
+             version ".tar.gz"))
+       (sha256
+        (base32
+         "168cj96974n2z0igin6j1ic1m45zyic7nm5ark7frq8j78rrx4zn"))))
+    (build-system gnu-build-system)
+    (arguments
+     `(#:tests? #f ; no "check" target
+       #:make-flags (list (string-append "prefix=" (assoc-ref %outputs "out")))
+       #:phases (modify-phases %standard-phases (delete 'configure))))
+    (inputs
+     `(("gawk" ,gawk)))
+    (home-page "https://github.com/mvertes/txt2man")
+    (synopsis "Convert text to man page")
+    (description "Txt2man converts flat ASCII text to man page format.")
+    (license gpl2+)))
