@@ -1,9 +1,12 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2013, 2014 Eric Bavier <bavier@member.fsf.org>
 ;;; Copyright © 2015 Mark H Weaver <mhw@netris.org>
-;;; Copyright © 2015 Efraim Flashner <efraim@flashner.co.il>
+;;; Copyright © 2015, 2016, 2017 Efraim Flashner <efraim@flashner.co.il>
 ;;; Copyright © 2016 Pjotr Prins <pjotr.guix@thebird.nl>
 ;;; Copyright © 2016 Andreas Enge <andreas@enge.fr>
+;;; Copyright © 2016 Ricardo Wurmus <rekado@elephly.net>
+;;; Copyright © 2016 Ben Woodcroft <donttrustben@gmail.com>
+;;; Copyright © 2017 Rutger Helling <rhelling@mykolab.com>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -28,6 +31,7 @@
   #:use-module (gnu packages)
   #:use-module (gnu packages admin)
   #:use-module (gnu packages autotools)
+  #:use-module (gnu packages base)
   #:use-module (gnu packages freeipmi)
   #:use-module (gnu packages linux)
   #:use-module (gnu packages mpi)
@@ -42,7 +46,7 @@
 (define-public parallel
   (package
     (name "parallel")
-    (version "20160222")
+    (version "20171122")
     (source
      (origin
       (method url-fetch)
@@ -50,10 +54,35 @@
                           version ".tar.bz2"))
       (sha256
        (base32
-        "1sjmvinwr9j2a0jdk9y9nf2x4hhzcbl529slkwpz0vva0cwybywd"))))
+        "19l2r8zzh8jyb76r3ksfq9y3r8fg07xw31h0dq1d7jpf73fnmd21"))))
     (build-system gnu-build-system)
-    (inputs `(("perl" ,perl)))
-    (home-page "http://www.gnu.org/software/parallel/")
+    (arguments
+     `(#:phases
+       (modify-phases %standard-phases
+         (add-after 'unpack 'patch-bin-sh
+           (lambda _
+             (for-each
+              (lambda (file)
+                (substitute* file
+                  ;; Patch hard coded '/bin/sh' in the lin ending in:
+                  ;; $Global::shell = $ENV{'PARALLEL_SHELL'} ||
+                  ;;  parent_shell($$) || $ENV{'SHELL'} || "/bin/sh";
+                  (("/bin/sh\\\";\n$") (string-append (which "sh") "\";\n"))
+                  ;; Patch call to 'ps' and 'perl' commands.
+                  ((" ps ") (string-append " " (which "ps") " "))
+                  ((" perl -") (string-append " " (which "perl") " -"))))
+              (list "src/parallel" "src/sem"))
+             #t))
+         (add-after 'install 'post-install-test
+           (lambda* (#:key outputs #:allow-other-keys)
+             (zero? (system* (string-append
+                              (assoc-ref outputs "out") "/bin/parallel")
+                             "echo"
+                             ":::" "1" "2" "3")))))))
+    (inputs
+     `(("perl" ,perl)
+       ("procps" ,procps)))
+    (home-page "https://www.gnu.org/software/parallel/")
     (synopsis "Build and execute command lines in parallel")
     (description
      "GNU Parallel is a tool for executing shell jobs in parallel using one
@@ -64,18 +93,17 @@ and they are executed on lists of files, hosts, users or other items.")
 (define-public slurm
   (package
    (name "slurm")
-   (version "15.08.7.1")
+   (version "16.05.11")
    (source (origin
             (method url-fetch)
             (uri (string-append
-                  "https://github.com/SchedMD/slurm/archive/slurm-"
-                  (string-join (string-split version #\.) "-") ".tar.gz"))
-            (file-name (string-append name "-" version ".tar.gz"))
+                  "https://download.schedmd.com/slurm/slurm-"
+                  version ".tar.bz2"))
             (sha256
              (base32
-              "1rmi35l4img00dr4vic8cv8s7b6n1yx1mkq2s7kjf5hvqdh6s2ki"))
-            (patches (list
-             (search-patch "slurm-configure-remove-nonfree-contribs.patch")))
+              "0c63mvh13wsp6jlydaz98v35iwg53mk94ynpx9dqn2z4gl53k5y7"))
+            (patches (search-patches
+                      "slurm-configure-remove-nonfree-contribs.patch"))
             (modules '((guix build utils)))
             (snippet
              '(begin
@@ -85,7 +113,7 @@ and they are executed on lists of files, hosts, users or other items.")
    ;; in particular mysql and gtk+.
    (inputs `(("expect" ,expect)
              ("freeipmi" ,freeipmi)
-             ("hwloc" ,hwloc)
+             ("hwloc" ,hwloc "lib")
              ("json-c" ,json-c)
              ("linux-pam" , linux-pam)
              ("munge" ,munge)
@@ -108,9 +136,8 @@ and they are executed on lists of files, hosts, users or other items.")
             (string-append "--with-ssl=" (assoc-ref %build-inputs "openssl")))
       #:phases
       (modify-phases %standard-phases
-       (add-before
-        'configure 'autogen
-        (lambda _ (zero? (system* "autoconf"))))))) ; configure.ac was patched
+        (add-after 'unpack 'autogen
+          (lambda _ (zero? (system* "autoconf"))))))) ; configure.ac was patched
    (home-page "http://slurm.schedmd.com/")
    (synopsis "Workload manager for cluster computing")
    (description
@@ -121,3 +148,30 @@ framework for starting, executing, and monitoring work (typically a parallel
 job) on a set of allocated nodes, and arbitrates contention for resources
 by managing a queue of pending work.")
    (license license:gpl2+)))
+
+(define-public slurm-drmaa
+  (package
+    (name "slurm-drmaa")
+    (version "1.0.7")
+    (source (origin
+              (method url-fetch)
+              (uri "http://apps.man.poznan.pl/trac/slurm-drmaa/downloads/9")
+              (file-name (string-append name "-" version ".tar.gz"))
+              (sha256
+               (base32
+                "0grw55hmny2mc4nc0y1arnvxd2k0dcdfn476kzs180fibjxgfw14"))))
+    (build-system gnu-build-system)
+    (inputs
+     `(("slurm" ,slurm)))
+    (native-inputs
+     `(("which" ,which)))
+    (home-page "http://apps.man.poznan.pl/trac/slurm-drmaa")
+    (synopsis "Distributed resource management application API for SLURM")
+    (description
+     "PSNC DRMAA for Simple Linux Utility for Resource Management (SLURM) is
+an implementation of Open Grid Forum DRMAA 1.0 (Distributed Resource
+Management Application API) specification for submission and control of jobs
+to SLURM.  Using DRMAA, grid applications builders, portal developers and ISVs
+can use the same high-level API to link their software with different
+cluster/resource management systems.")
+    (license license:gpl3+)))
