@@ -1,5 +1,5 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2012, 2013 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2012, 2013, 2014, 2016 Ludovic Courtès <ludo@gnu.org>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -30,23 +30,30 @@
 (define-public %pkg-config
   (package
    (name "pkg-config")
-   (version "0.27.1")
+   (version "0.29.2")
    (source (origin
             (method url-fetch)
-            (uri (string-append
-                  "http://pkgconfig.freedesktop.org/releases/pkg-config-"
-                  version ".tar.gz"))
+            (uri (list
+                  (string-append
+                   "http://fossies.org/linux/misc/pkg-config-" version
+                   ".tar.gz")
+
+                  ;; FIXME: The following URL redirects to HTTPS, which
+                  ;; creates bootstrapping problems:
+                  ;; <http://bugs.gnu.org/22774>.
+                  (string-append
+                   "http://pkgconfig.freedesktop.org/releases/pkg-config-"
+                   version ".tar.gz")))
             (sha256
              (base32
-              "05wc5nwkqz7saj2v33ydmz1y6jdg659dll4jjh91n41m63gx0qsg"))))
+              "14fmwzki1rlz8bs2p810lk6jqdxsk966d8drgsjmi54cd00rrikg"))))
    (build-system gnu-build-system)
    (arguments `(#:configure-flags '("--with-internal-glib")))
    (native-search-paths
     (list (search-path-specification
            (variable "PKG_CONFIG_PATH")
-           (directories '("lib/pkgconfig" "lib64/pkgconfig"
-                          "share/pkgconfig")))))
-   (home-page "http://www.freedesktop.org/wiki/Software/pkg-config")
+           (files '("lib/pkgconfig" "lib64/pkgconfig" "share/pkgconfig")))))
+   (home-page "https://www.freedesktop.org/wiki/Software/pkg-config")
    (license gpl2+)
    (synopsis "Helper tool used when compiling applications and libraries")
    (description
@@ -71,13 +78,11 @@ instance.")))
        #:builder (begin
                    (use-modules (guix build utils))
 
-                   (let* ((out  (assoc-ref %outputs "out"))
-                          (bin  (string-append out "/bin"))
-                          (prog (string-append ,target "-pkg-config"))
-                          (native
-                           (string-append
-                            (assoc-ref %build-inputs "pkg-config")
-                            "/bin/pkg-config")))
+                   (let* ((in     (assoc-ref %build-inputs "pkg-config"))
+                          (out    (assoc-ref %outputs "out"))
+                          (bin    (string-append out "/bin"))
+                          (prog   (string-append ,target "-pkg-config"))
+                          (native (string-append in "/bin/pkg-config")))
 
                      (mkdir-p bin)
 
@@ -85,7 +90,13 @@ instance.")))
                      ;; This satisfies the pkg.m4 macros, which use
                      ;; AC_PROG_TOOL to determine the `pkg-config' program
                      ;; name.
-                     (symlink native (string-append bin "/" prog))))))
+                     (symlink native (string-append bin "/" prog))
+
+                     ;; Also make 'pkg.m4' available, some packages might
+                     ;; expect it.
+                     (mkdir-p (string-append out "/share"))
+                     (symlink (string-append in "/share/aclocal")
+                              (string-append out "/share/aclocal"))))))
     (native-inputs `(("pkg-config" ,%pkg-config)))
 
     ;; Ignore native inputs, and set `PKG_CONFIG_PATH' for target inputs.
