@@ -30,6 +30,7 @@
   #:use-module (gnu packages autotools)
   #:use-module (gnu packages databases)
   #:use-module (gnu packages boost)
+  #:use-module (gnu packages check)
   #:use-module (gnu packages compression)
   #:use-module (gnu packages curl)
   #:use-module (gnu packages cyrus-sasl)
@@ -41,7 +42,9 @@
   #:use-module (gnu packages perl)
   #:use-module (gnu packages pkg-config)
   #:use-module (gnu packages python)
+  #:use-module (gnu packages python-web)
   #:use-module (gnu packages qt)
+  #:use-module (gnu packages time)
   #:use-module (gnu packages tls)
   #:use-module (gnu packages xml))
 
@@ -53,6 +56,8 @@
              (method url-fetch)
              (uri (string-append "http://download.librdf.org/source/" name
                                  "-" version ".tar.gz"))
+             (patches
+              (search-patches "raptor2-heap-overflow.patch"))
              (sha256
               (base32
                "1vc02im4mpc28zxzgli68k6j0dakh0k3s389bm436yvqajxg19xd"))))
@@ -125,26 +130,26 @@ Java Lucene text search engine API to C++.")
                 "18p2flb2sv2hq6w2qkd29z9c7knnwqr3f12i2srshlzx6vwkm05s"))))
     (build-system gnu-build-system)
     (arguments
-     '(#:phases (alist-cons-after
-                 'remove-out-of-tree-references 'autoreconf
-                 (lambda _
-                   (zero? (system* "autoreconf" "-vfi")))
-                 (alist-cons-after
-                  'unpack 'remove-out-of-tree-references
-                  (lambda _
-                    ;; remove symlinks to files in /usr/
-                    (delete-file-recursively "m4")
-                    (for-each delete-file '("config.guess"
-                                            "config.sub"
-                                            "depcomp"
-                                            "install-sh"
-                                            "ltmain.sh"
-                                            "missing"))
-                    ;; remove_test depends on an out-of-tree RDF file
-                    (substitute* "examples/Makefile.am"
-                      (("instances_test remove_test") "instances_test")
-                      (("\\$\\(TESTS\\) remove_test") "$(TESTS)")))
-                  %standard-phases))))
+     '(#:phases
+       (modify-phases %standard-phases
+         (add-after 'unpack 'remove-out-of-tree-references
+           (lambda _
+             ;; remove symlinks to files in /usr/
+             (delete-file-recursively "m4")
+             (for-each delete-file '("config.guess"
+                                     "config.sub"
+                                     "depcomp"
+                                     "install-sh"
+                                     "ltmain.sh"
+                                     "missing"))
+             ;; remove_test depends on an out-of-tree RDF file
+             (substitute* "examples/Makefile.am"
+               (("instances_test remove_test") "instances_test")
+               (("\\$\\(TESTS\\) remove_test") "$(TESTS)"))
+             #t))
+         (add-after 'remove-out-of-tree-references 'autoreconf
+           (lambda _
+             (zero? (system* "autoreconf" "-vfi")))))))
     (inputs
      `(("raptor" ,raptor2)
        ("cyrus-sasl" ,cyrus-sasl)
@@ -233,15 +238,14 @@ and triple stores.")
 (define-public serd
   (package
     (name "serd")
-    (version "0.22.0")
+    (version "0.26.0")
     (source (origin
              (method url-fetch)
              (uri (string-append "http://download.drobilla.net/serd-"
-                                 version
-                                 ".tar.bz2"))
+                                 version ".tar.bz2"))
              (sha256
               (base32
-               "1lmgca2s6r7ysakcld2lrj06bgn0pr45c51b47k3apxpnj3h40vv"))))
+               "164j43am4hka2vbzw4n52zy7rafgp6kmkgbcbvap368az644mr73"))))
     (build-system waf-build-system)
     (arguments
      `(#:tests? #f ; no check target
@@ -267,15 +271,14 @@ ideal (e.g. in LV2 implementations or embedded applications).")
 (define-public sord
   (package
     (name "sord")
-    (version "0.14.0")
+    (version "0.16.0")
     (source (origin
              (method url-fetch)
              (uri (string-append "http://download.drobilla.net/sord-"
-                                 version
-                                 ".tar.bz2"))
+                                 version ".tar.bz2"))
              (sha256
               (base32
-               "16piw1s3l95cf9x3rdcplp9l52k2lxq8drwg2q50ygm4avndhmkn"))))
+               "0nh3i867g9z4kdlnk82cg2kcw8r02qgifxvkycvzb4vfjv4v4g4x"))))
     (build-system waf-build-system)
     (arguments
      `(#:tests? #f ; no check target
@@ -315,6 +318,18 @@ ideal (e.g. in LV2 implementations or embedded applications).")
           (base32
             "0kvaf332cqbi47rqzlpdx4mbkvw12mkrzkj8n9l19wk713d4py9w"))))
     (build-system python-build-system)
+    (arguments
+     '(;; FIXME: Three test failures. Try uncommenting the below next update.
+       #:tests? #f))
+       ;; #:phases
+       ;; (modify-phases %standard-phases
+       ;;   (replace 'check
+       ;;     (lambda _
+       ;;       ;; Run tests from the build directory so python3 only
+       ;;       ;; sees the installed 2to3 version.
+       ;;       (zero? (system* "nosetests" "--where=./build/src")))))
+    (native-inputs
+     `(("python-nose" ,python-nose)))
     (propagated-inputs
       `(("python-html5lib" ,python-html5lib)
         ("python-isodate" ,python-isodate)
@@ -329,12 +344,4 @@ powerful language for representing information.")
                         "See LICENSE in the distribution."))))
 
 (define-public python2-rdflib
-  (let ((base (package-with-python2 python-rdflib)))
-    (package
-      (inherit base)
-      (inputs
-        (append (package-inputs base)
-                `(("python2-nose" ,python2-nose))))
-      (arguments
-        `(#:python ,python-2
-          #:tests? #f))))) ; 3 tests fail, also outside Guix
+  (package-with-python2 python-rdflib))
