@@ -1,5 +1,8 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2013, 2014 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2015 Andreas Enge <andreas@enge.fr>
+;;; Copyright © 2015 Mark H Weaver <mhw@netris.org>
+;;; Copyright © 2016 Efraim Flashner <efraim@flashner.co.il>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -28,28 +31,38 @@
 (define-public valgrind
   (package
     (name "valgrind")
-    (version "3.10.0")
+    (version "3.12.0")
     (source (origin
              (method url-fetch)
              (uri (string-append "http://valgrind.org/downloads/valgrind-"
                                  version ".tar.bz2"))
              (sha256
               (base32
-               "1jgd42vsx0bcblp91bd61hd5wpy0gghh09wxgm65m666vy17y103"))))
+               "18bnrw9b1d55wi1wnl68n25achsp9w48n51n1xw4fwjjnaal7jk7"))
+             (patches (search-patches "valgrind-enable-arm.patch"))))
     (build-system gnu-build-system)
+    (outputs '("doc"                              ;16 MB
+               "out"))
     (arguments
-     '(#:phases (alist-cons-after
-                 'install 'patch-suppression-files
-                 (lambda* (#:key outputs #:allow-other-keys)
-                   ;; Don't assume the FHS.
-                   (let* ((out (assoc-ref outputs "out"))
-                          (dir (string-append out "/lib/valgrind")))
-                     (substitute* (find-files dir "\\.supp$")
-                       (("obj:/lib") "obj:*/lib")
-                       (("obj:/usr/X11R6/lib") "obj:*/lib")
-                       (("obj:/usr/lib") "obj:*/lib"))
-                     #t))
-                 %standard-phases)))
+     '(#:phases
+       (modify-phases %standard-phases
+         (add-after 'install 'patch-suppression-files
+           (lambda* (#:key outputs #:allow-other-keys)
+             ;; Don't assume the FHS.
+             (let* ((out (assoc-ref outputs "out"))
+                    (dir (string-append out "/lib/valgrind")))
+               (substitute* (find-files dir "\\.supp$")
+                 (("obj:/lib") "obj:*/lib")
+                 (("obj:/usr/X11R6/lib") "obj:*/lib")
+                 (("obj:/usr/lib") "obj:*/lib"))
+               #t)))
+         (add-after 'install 'install-doc
+           (lambda* (#:key outputs #:allow-other-keys)
+             (let ((orig (format #f "~a/share/doc" (assoc-ref outputs "out")))
+                   (dest (format #f "~a/share" (assoc-ref outputs "doc"))))
+               (mkdir-p dest)
+               (rename-file orig dest)
+               #t))))))
     (inputs `(;; GDB is needed to provide a sane default for `--db-command'.
               ("gdb" ,gdb)))
     (native-inputs `(("perl" ,perl)))
