@@ -39,7 +39,7 @@
                "04n2lac0vgpv8zsn7nmb50hf3qb56pmj90dmwnivg09gyrf1x92j"))))
     (build-system gnu-build-system)
     (inputs
-     `(("gtk+", gtk+-2)
+     `(("gtk+" ,gtk+-2)
        ("inputproto" ,inputproto)
        ("libx11" ,libx11)
        ("libxext" ,libxext)
@@ -50,7 +50,7 @@
        ("xproto" ,xproto)))
     (native-inputs
       `(("pkg-config" ,pkg-config)))
-    (home-page "http://www.gnu.org/software/xnee/")
+    (home-page "https://www.gnu.org/software/xnee/")
     (synopsis "Record, replay and distribute user actions under X11")
     (description
      "GNU Xnee is a program that can record, replay and distribute user
